@@ -1,12 +1,13 @@
 from revpayment.settings import api_settings
 from revpayment.models import RedirectState
+from revpayment import exceptions
 from django.shortcuts import redirect
 from rest_framework import response
 from django.conf import settings
 import requests
 import json
 import jwt
-from jwcrypto.jwt import JWK
+from revjwt import decode
 
 
 class CheckoutSDK:
@@ -16,6 +17,7 @@ class CheckoutSDK:
         ('credit', api_settings.DEFAULT_CREDIT_CLASS),
     )
     handler_class = api_settings.HANDLER_CLASS
+    fail_url = api_settings.DEFAULT_CHECKOUTFAIL_URL
     redirect_url = api_settings.DEFAULT_REDIRECT_URL
     redirect_query = api_settings.DEFAULT_REDIRECT_QUERY
 
@@ -37,16 +39,20 @@ class CheckoutSDK:
                 c[1] for c in self.payment_classes if c[0] == self.payment_type][0]
             return payment_class
         except IndexError:
-            raise ValueError('payment_type is not valid')
+            valids = [payment[0] for payment in self.payment_classes]
+            raise exceptions.InvalidPaymentType(valids=valids, invalid=self.payment_type)
 
     def get_payment(self):
         payment_class = self.get_payment_class()
         return payment_class(buyer=self.buyer, payment_subtype=self.payment_subtype, cart=self.cart, order_id=self.state.order_id, order_type=self.state.order_type)
 
     def checkout(self):
-        payment = self.get_payment()
-        result = payment.checkout()
-        return redirect(result['url'])
+        try:
+            payment = self.get_payment()
+            result = payment.checkout()
+            return redirect(result['url'])
+        except exceptions.PaymentException as e:
+            return redirect(f'{self.fail_url}?error={e.error}&error_detail={e.error_detail}')
 
     def callback(self, data):
         payment = self.get_payment()
@@ -63,30 +69,13 @@ class BaseCertificate:
     default_aud = settings.CLIENT_ID
 
     def __init__(self, cert):
-        self.unverified = jwt.decode(cert, verify=False)
-        self.headers = jwt.get_unverified_header(cert)
         self.cert = cert
 
     def get_order_id(self, sub):
         raise NotImplementedError
 
-    def get_key_url(self):
-        iss = self.unverified['iss']
-        host = iss.split('.')[0]
-        if host[-4:] in ['-stg', '-dev']:
-            return 'https://auth-stg.revtel-api.com/v3/certs'
-        return 'https://auth.revtel-api.com/v3/certs'
-
-    def get_pub_key(self):
-        url = self.get_key_url()
-        jwks = requests.get(url).json()['keys']
-        key = [jwk for jwk in jwks if jwk['kid'] == self.headers['kid']][0]
-        real_key = JWK.from_json(json.dumps(key))
-        return real_key.export_to_pem()
-
     def verify(self):
-        pub = self.get_pub_key()
-        decoded = jwt.decode(self.cert, algorithms='RS256', key=pub, verify=True, audience=self.default_aud)
+        decoded = decode(self.cert, verify=True, audience=self.default_aud)
         if decoded['typ'] != 'certificate':
             raise
 
